@@ -272,35 +272,11 @@ class Layer(Widget, InteractMixin):
 
     bottom = Bool().tag(sync=True)
     options = List(trait=Unicode).tag(sync=True)
-    opacity = Float(1.0, min=0.0, max=1.0).tag(sync=True)
 
     @default('options')
     def _default_options(self):
         return [name for name in self.traits(o=True)]
 
-    _map = None
-
-    visible = Bool(False)
-
-    @observe('visible')
-    def _update_visible(self, change):
-        was_visible = change['old']
-        will_visible = change['new']
-        if self._map is None:
-            # If Map.add_layer has never been called raise
-            raise LayerException('Map.add_layer() must be called first')
-        else:
-            # Map.add_layer() was called
-            if (not was_visible) and will_visible:
-                # Only add if we aren't already in self._map.layers
-                if self.model_id not in self._map.layer_ids:
-                    self._map.add_layer(self)
-            # Map.remove_layer() was called
-            elif was_visible and (not will_visible):
-                # Only remove if we are in self._map.layers
-                if self.model_id in self._map.layer_ids:
-                    self._map.remove_layer(self)
-
 
 class UILayer(Layer):
     _view_name = Unicode('LeafletUILayerView').tag(sync=True)
@@ -322,6 +298,9 @@ class Marker(UILayer):
     alt = Unicode().tag(sync=True, o=True)
     rise_on_hover = Bool(False).tag(sync=True, o=True)
 
+    opacity = Float(1.0, min=0.0, max=1.0).tag(sync=True)
+    visible = Bool(True).tag(sync=True)
+
     rise_offset = Int(250).tag(sync=True, o=True)
 
     _move_callbacks = Instance(CallbackDispatcher, ())
@@ -346,6 +325,9 @@ class RasterLayer(Layer):
     _view_name = Unicode('LeafletRasterLayerView').tag(sync=True)
     _model_name = Unicode('LeafletRasterLayerModel').tag(sync=True)
 
+    opacity = Float(1.0, min=0.0, max=1.0).tag(sync=True)
+    visible = Bool(True).tag(sync=True)
+
 
 class TileLayer(RasterLayer):
     _view_name = Unicode('LeafletTileLayerView').tag(sync=True)
@@ -524,28 +506,6 @@ class Control(Widget):
     def _default_options(self):
         return [name for name in self.traits(o=True)]
 
-    _map = None
-
-    visible = Bool(False)
-
-    @observe('visible')
-    def _update_visible(self, change):
-        was_visible = change['old']
-        will_visible = change['new']
-        if self._map is None:
-            # If Map.add_control has never been called raise
-            raise ControlException('Map.add_control() must be called first')
-        else:
-            # Map.add_control() was called
-            if (not was_visible) and will_visible:
-                # Only add if we aren't already in self._map.controls
-                if self.model_id not in self._map.control_ids:
-                    self._map.add_control(self)
-            # Map.remove_controls() was called
-            elif was_visible and (not will_visible):
-                # Only remove if we are in self._map.controls
-                if self.model_id in self._map.control_ids:
-                    self._map.remove_control(self)
 
 class LayersControl(Control):
     _view_name = Unicode('LeafletLayersControlView').tag(sync=True)
@@ -731,23 +691,17 @@ class Map(DOMWidget, InteractMixin):
     def add_layer(self, layer):
         if layer.model_id in self.layer_ids:
             raise LayerException('layer already on map: %r' % layer)
-        layer._map = self
         self.layers = tuple([l for l in self.layers] + [layer])
-        layer.visible = True
 
     def remove_layer(self, layer):
         if layer.model_id not in self.layer_ids:
             raise LayerException('layer not on map: %r' % layer)
         self.layers = tuple([l for l in self.layers if l.model_id != layer.model_id])
-        layer.visible = False
 
     def substitute_layer(self, old, new):
         if old.model_id not in self.layer_ids:
             raise LayerException('layer not on map: %r' % layer)
-        self.layers = tuple([new if l.model_id == old.model_id else old for l in self.layers])
-        old.visible = False
-        new._map = self
-        new.visible = True
+        self.layers = tuple([new if l.model_id == old.model_id else l for l in self.layers])
 
     def clear_layers(self):
         self.layers = ()
@@ -770,15 +724,12 @@ class Map(DOMWidget, InteractMixin):
     def add_control(self, control):
         if control.model_id in self.control_ids:
             raise ControlException('control already on map: %r' % control)
-        control._map = self
         self.controls = tuple([c for c in self.controls] + [control])
-        control.visible = True
 
     def remove_control(self, control):
         if control.model_id not in self.control_ids:
             raise ControlException('control not on map: %r' % control)
         self.controls = tuple([c for c in self.controls if c.model_id != control.model_id])
-        control.visible = False
 
     def clear_controls(self):
         self.controls = ()
